local mvec3_set = mvector3.set
local mvec3_add = mvector3.add
local mvec3_dot = mvector3.dot
local mvec3_sub = mvector3.subtract
local mvec3_mul = mvector3.multiply
local mvec3_norm = mvector3.normalize
local mvec3_dir = mvector3.direction
local mvec3_set_l = mvector3.set_length
local mvec3_len = mvector3.length
local math_clamp = math.clamp
local math_lerp = math.lerp
local tmp_vec1 = Vector3()
local tmp_vec2 = Vector3()
local tmp_rot1 = Rotation()

XM25SnpExpBulletBase = XM25SnpExpBulletBase or class(InstantExplosiveBulletBase)
--XM25SnpExpBulletBase.id = "InstantExplosiveBulletBase"
XM25SnpExpBulletBase.CURVE_POW = tweak_data.upgrades.explosive_bullet.curve_pow
XM25SnpExpBulletBase.PLAYER_DMG_MUL = 0.05
XM25SnpExpBulletBase.RANGE = 250
XM25SnpExpBulletBase.EFFECT_PARAMS = {
	sound_event = "gl_explode",		--"grenade_electric_explode"
	effect = "effects/payday2/particles/explosions/bag_explosion",
    	on_unit = true,
    	sound_muffle_effect = true,
    	idstr_decal = Idstring("explosion_round"),
    	idstr_effect = Idstring("")
}

function XM25SnpExpBulletBase:on_collision(col_ray, weapon_unit, user_unit, damage, blank)
	local hit_unit = col_ray.unit

	if not hit_unit:character_damage() or not hit_unit:character_damage()._no_blood then
		self:play_impact_sound_and_effects(weapon_unit, col_ray)
	end

	if not blank then
		mvec3_set(tmp_vec1, col_ray.position)
		mvec3_set(tmp_vec2, col_ray.ray)
		mvec3_norm(tmp_vec2)
		mvec3_mul(tmp_vec2, 10)
		mvec3_sub(tmp_vec1, tmp_vec2)

		local network_damage = math.ceil(damage * 250)
		damage = network_damage / 163.84

		if Network:is_server() then
			self:on_collision_server(tmp_vec1, col_ray.normal, damage, user_unit, weapon_unit, managers.network:session():local_peer():id())
		else
			self:on_collision_server(tmp_vec1, col_ray.normal, damage, user_unit, weapon_unit, managers.network:session():local_peer():id())
		end

		if hit_unit:damage() and col_ray.body:extension() and col_ray.body:extension().damage then
			local sync_damage = not blank and hit_unit:id() ~= -1

			if sync_damage then
				local normal_vec_yaw, normal_vec_pitch = self._get_vector_sync_yaw_pitch(col_ray.normal, 128, 64)
				local dir_vec_yaw, dir_vec_pitch = self._get_vector_sync_yaw_pitch(col_ray.ray, 128, 64)

				managers.network:session():send_to_peers_synched("sync_body_damage_bullet", col_ray.body, user_unit:id() ~= -1 and user_unit or nil, normal_vec_yaw, normal_vec_pitch, col_ray.position, dir_vec_yaw, dir_vec_pitch, math.min(16384, network_damage))
			end

			local local_damage = not blank or hit_unit:id() == -1

			if local_damage then
				col_ray.body:extension().damage:damage_bullet(user_unit, col_ray.normal, col_ray.position, col_ray.ray, 1)
				col_ray.body:extension().damage:damage_damage(user_unit, col_ray.normal, col_ray.position, col_ray.ray, damage)

				if alive(weapon_unit) and weapon_unit:base().categories and weapon_unit:base():categories() then
					for _, category in ipairs(weapon_unit:base():categories()) do
						col_ray.body:extension().damage:damage_bullet_type(category, user_unit, col_ray.normal, col_ray.position, col_ray.ray, 1)
					end
				end
			end
		end

		return {
			variant = "explosion",
			col_ray = col_ray
		}
	end

	return nil
end

function XM25SnpExpBulletBase:play_impact_sound_and_effects(weapon_unit, col_ray)
	managers.game_play_central:play_impact_sound_and_effects(self:_get_sound_and_effects_params(weapon_unit, col_ray, false))
	-- self:_spawn_environment_fire(normal, col_ray)
end

function XM25SnpExpBulletBase:_spawn_environment_fire(normal, col_ray)
	local position = col_ray.position
	local rotation = Rotation(0, 0, 0)

	local data = tweak_data.env_effect:incendiary_fire()

	-- local data = tweak_data.env_effect:trip_mine_fire()
	-- local data = tweak_data.env_effect:molotov_fire()
	data.burn_duration = 10
	data.range = 130

	EnvironmentFire.spawn(position, rotation, data, normal, self._thrower_unit, 0, 1)

	self.burn_stop_time = TimerManager:game():time() + 10
end



